;;; Quick and dirty bindings to OSMesa. Requires loading libOSMesa.so
;;; before loading cl-opengl. Leaks memory.
;;;
;;;   (require :cffi)
;;;   (cffi:load-foreign-library "libOSMesa.so")
;;;   (require :cl-opengl)
;;;   <then load this file>
;;;   (setq %gl:*gl-get-proc-address* 'osmesa:get-proc-address) ; optional
;;;   (osmesa:make-current (osmesa:create-context))
;;;   (gl:get-string :version) => "2.1 Mesa 7.4"

(defpackage #:osmesa
  (:use #:cffi #:common-lisp)
  (:export #:create-context #:destroy-context #:make-current
           #:get-proc-address))

(in-package #:osmesa)

(defctype context :pointer)

(defcenum format-enum
  (:color-index #.(cffi:foreign-enum-value '%gl:enum :color-index))
  (:rgba #.(cffi:foreign-enum-value '%gl:enum :rgba))
  (:bgra 1)
  (:argb 2)
  (:rgb #.(cffi:foreign-enum-value '%gl:enum :rgb))
  (:bgr 4)
  (:rgb-565 5))

(defcfun ("OSMesaCreateContext" %create-context) context
  (format format-enum)
  (share-list context))

(defun create-context (&optional (format :rgba) share)
  (declare (ignore share))
  (assert (eq format :rgba))
  (%create-context format (null-pointer)))

(defcfun ("OSMesaDestroyContext" destroy-context) :void
  (ctx context))

(defcfun ("OSMesaMakeCurrent" %make-current) %gl:boolean
  (ctx context)
  (buffer :pointer)
  (type %gl:enum)
  (width %gl:sizei)
  (height %gl:sizei))

(defun make-current (context &key (width 1) (height 1))
  (%make-current context (foreign-alloc :uint32 :count (* width height 4))
                 :unsigned-byte width height))

(defcfun ("OSMesaGetProcAddress" get-proc-address) :pointer
  (func-name :string))
